#!/usr/bin/env python
"""
_ListFilesetByTask_

Oracle implementation of Fileset.ListFilesetByTask
"""





from WMCore.WMBS.MySQL.Fileset.ListFilesetByTask import ListFilesetByTask as ListFilesetByTaskMySQL

class ListFilesetByTask(ListFilesetByTaskMySQL):
    pass
