#!/usr/bin/env python
"""
_SetPhEDExStatus_

Oracle implementation of DBSBufferFiles.SetPhEDExStatus
"""

from WMComponent.DBS3Buffer.MySQL.DBSBufferFiles.SetPhEDExStatus \
      import SetPhEDExStatus as MySQLSetPhEDExStatus

class SetPhEDExStatus(MySQLSetPhEDExStatus):
    """
    _SetPhEDExStatus_

    Oracle implementation of SetPhEDExStatus
    """
    pass
