#!/usr/bin/env python
"""
_IsCompleteOnRun_

Oracle implementation of Subscriptions.IsCompleteOnRun
"""

from WMCore.WMBS.MySQL.Subscriptions.IsCompleteOnRun import IsCompleteOnRun as IsCompleteOnRunMySQL

class IsCompleteOnRun(IsCompleteOnRunMySQL):
    pass
